#!/usr/bin/env python
from time import time
from collections import deque
from weakref import WeakValueDictionary

from twisted.internet import reactor

from omegle import Stranger, HTTP
from core import CBDictInterface
from event import isEvent, mkIterableSequence, Transmogrifier, ReactorEvent, IDLE_TIMEOUT, NULL_EVENT
from listener import InteractiveViewport


class TrollReactor(CBDictInterface):
    """Base class for Omegle API.
    """
    def __init__(self, transmog=Transmogrifier(), listen=InteractiveViewport(), n=2, refresh=1.5):
        # Independent setup
        super(TrollReactor, self).__init__()
        self.listeners = WeakValueDictionary()
        # Argument assignment
        self.eventQueue = deque()
        self.connectTransmogrifier(transmog)
        self.addListeners(listen)
        self._n = n
        self.refresh = refresh

        self._allConnected = False
        self.idleTime = None
        self.initializeStrangers()  # Now we wait to receive idSet events

    def connectTransmogrifier(self, transmog):
        self.transmogrifier = transmog
        self.transmogrifier.connect(self.eventQueue)

    def initializeStrangers(self):
        self._volatile = dict((Stranger(reactor, self, HTTP), None) for _ in xrange(self._n))
        self._waiting = len(self._volatile.keys())
        self.strangers = {}
        self.idleTime = time()
        self._allConnected = False

    def multicastDisconnect(self, ids):
        """Announce disconnect for a group of strangers.

        ids : iterable
            id strings of strangers from whom to politely disconnect.
        """
        for i in ids:
            self.strangers[i].announceDisconnect()

    def restart(self):
        self.strangers.clear()
        self._allConnected = False
        self.initializeStrangers()

    def pumpEvents(self):
        for id_ in self.strangers:
            self.strangers[id_].getEventsPage()

        reactor.callLater(self.refresh, self.pumpEvents)

    def on_idSet(self, ev):
        for s in self._volatile:
            if s.id == ev.id:  # we have the stranger that notified
                self.strangers[s.id] = s  # move to {id: stranger} dict
                self._waiting -= 1

        assert self._waiting >= 0, "Too many stranger IDs"
        if self._waiting == 0:
            self._allConnected = True
            self.idleTime = time()
            self.pumpEvents()

    def on_error(self, ev):
        pass  #  This is where we handle RECAPCHA

    def addListeners(self, listeners):
        """Add a listener or group of listeners to the reactor.

        listeners : CBDictInterface instance or iterable
        """
        listeners = mkIterableSequence(listeners)

        for listen in listeners:
            self.listeners[listen] = listen  # weak-value dict

    def removeListener(self, listener):
        self.listeners.pop(listener)

    def _processEventQueue(self):
        while len(self.eventQueue):
            ev = self.eventQueue.popleft()
            for listener in self.listeners:
                listener.notify(ev)

            self.notify(ev)

    def deltaIdleTime(self):
        return time() - self.idleTime

    def idle(self):
        """Respond to idle state.

        This function is run whenever feed encounters a null event, and
        does nothing by default.  Override to define functionality.
        """
        pass

    def feed(self, events):
        """Notify the TrollReactor of event(s).
        """
        if not events or events is NULL_EVENT:
            events = (NULL_EVENT,)
            self.idle()
        else:
            self.idleTime = time()

        if isEvent(events):  # if events is a single event
            events = (events,)

        self.transmogrifier(events)
        self._processEventQueue()


class Client(TrollReactor):
    """Extensible client for omegle.com.
    """
    def __init__(self, refresh=2):
        super(Client, self).__init__(listen=listen, n=1)
        self.refresh = 2
        self._connected = False
        self.stranger = None

        self.amTyping = False
        self.isTyping = False
        self.on_stoppedTyping = self.on_typing

        self.reset()

    def reset(self):
        self._connected = False
        self.stranger = None
        self.amTyping = False
        self.isTyping = False

    def getStranger(self):
        self.stranger = Stranger(reactor, self, HTTP)

    def on_idSet(self, ev):
        self._connected = True
        self.pumpEvents()

    def on_typing(self, ev):
        self.isTyping = not self.isTyping

    def on_strangerDisconnected(self, ev):
        self.reset()

    def disconnect(self):
        self.stranger.announceDisconnect()
        self.reset()

    def sendMessage(self, msg):
        self.stranger.sendMessage(msg)

    def pumpEvents(self):
        self.strangers.getEventsPage()
        reactor.callLater(self.refresh, self.pumpEvents)


class MiddleMan(TrollReactor):
    """Implementation of man-in-the-middle attack on two omegle users.
    """
    def __init__(self, transmog=Transmogrifier(), listen=InteractiveViewport(), idle=(0., 0.)):
        """Instantiate MiddleMan class

        transmog : Transmogrifier instance

        listen : single listener instance or iterable.
            listeners to assign.

        idle : tuple of floats
            idle[0] =   maximum time to wait for all connections (seconds)
            idle[1] =   maximum time to wait for an idle conversation to resume (seconds)
        """
        super(MiddleMan, self).__init__(transmog=transmog, listen=listen)
        self.max_connect_time, self.max_idle_time = idle
        self.on_stoppedTyping = self.on_typing

    def on_typing(self, ev):
        self.strangers[ev.id].toggle_typing()

    def on_strangerDisconnected(self, ev):
        active = (i for i in self.strangers if i != ev.id)
        self.multicastDisconnect(active)  # announce disconnect to everyone
        self.restart()

    def on_gotMessage(self, ev):
        for nonspeaker_id in (nspkr for nspkr in self.strangers if nspkr != ev.id):
            self.strangers[nonspeaker_id].sendMessage(ev.data)

    def idle(self):
        if not self.max_idle_time and not self.max_connect_time:
            return

        dIdle = self.deltaIdleTime()
        slow_conn = not self._allConnected and (dIdle > self.max_connect_time) and self.max_connect_time
        slow_conv = self._allConnected and (dIdle > self.max_idle_time) and self.max_idle_time

        if slow_conn or slow_conv:
            self.feed(ReactorEvent(IDLE_TIMEOUT, None))

    def on_timeout(self, ev):
        self.multicastDisconnect((i for i in self.strangers))
        self.restart()


class OMiner(object):
    """Data minig class
    """



#!/usr/bin/env python
from twisted.internet import reactor

from event import ID_SET, WAITING, CONNECTED, TYPING, STOPPED_TYPING, GOT_MESSAGE, DISCONNECTED, ERROR, IDLE_TIMEOUT, MESSAGE_MODIFIED

startTrolling = reactor.run
stopTrolling = reactor.stop

_doNothing = lambda x: None


class CBDictInterface(object):
    """Base class for all classes responding to OmegleEvents.
    """
    def __init__(self, callbackdict=None):
        self.callbacks = callbackdict or {
                            ID_SET: self.on_idSet,
                            WAITING: self.on_waiting,
                            CONNECTED: self.on_connected,
                            TYPING: self.on_typing,
                            STOPPED_TYPING: self.on_stoppedTyping,
                            GOT_MESSAGE: self.on_gotMessage,
                            DISCONNECTED: self.on_strangerDisconnected,
                            ERROR: self.on_error,
                            IDLE_TIMEOUT: self.on_timeout,
                            MESSAGE_MODIFIED: self.on_messageModified}

    def on_idSet(self, ev):
        pass

    def on_waiting(self, ev):
        pass

    def on_connected(self, ev):
        pass

    def on_typing(self, ev):
        pass

    def on_stoppedTyping(self, ev):
        pass

    def on_gotMessage(self, ev):
        pass

    def on_strangerDisconnected(self, ev):
        pass

    def on_error(self, ev):
        pass

    def on_timeout(self, ev):
        pass

    def on_messageModified(self, ev):
        pass

    def notify(self, ev):
        self.callbacks.get(ev.type, _doNothing)(ev)



#!/usr/bin/env python
from urllib import urlencode
import json
from random import choice
from traceback import print_stack
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO

from twisted.internet.defer import Deferred
from twisted.internet.protocol import Protocol
from twisted.web.client import Agent, FileBodyProducer
from twisted.web.http_headers import Headers

from event import OmegleEvent, ID_SET


class HTTP(Protocol):
    def __init__(self, response):
        self.response = response
        self.data = ''

    def dataReceived(self, bytes):
        self.data += bytes

    def connectionLost(self, reason):
        self.response.callback(self.data)


class Stranger(object):
    """Class to encapsulate I/O to an Omegle user.
    """
    _RESPONSE_OK = 200
    _ACTIONS = ('start', 'events', 'send','typing', 'disconnect')
    _api = dict(((a, 'http://omegle.com/' + a) for a in _ACTIONS))
    uagents = ["Mozilla/5.0 (Windows NT 6.1; WOW64; rv:14.0) Gecko/20100101 Firefox/14.0.1",
              "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; FDM; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 1.1.4322)",
              "Mozilla/5.0 (Windows; U; Windows NT 6.1; es-AR; rv:1.9) Gecko/2008051206 Firefox/3.0"]

    def __init__(self, reactor, troll, protocol):
        """reactor : twisted reactor instance
        protocol : class object
            Class of protocol being used
        """
        self.typing = False
        self.connected = False
        self.id = None

        self.reactor = reactor
        self.troll = troll  # exposes troll.notify
        self.protocol = protocol
        self.agent = choice(self.uagents)

        self._getStrangerID()

    def request(self, api_call, data):
        agent = Agent(self.reactor)
        header = {'User-Agent': [self.agent],
                  'content-type': ['application/x-www-form-urlencoded']}
        data = urlencode(data)
        return agent.request(
                'POST', self._api[api_call], Headers(header),
                FileBodyProducer(StringIO(data)))

    def getBody(self, response):
        body = Deferred()
        response.deliverBody(self.protocol(body))
        return body

    def _getStrangerID(self):
        d = self.request('start', '')
        # lvl 1
        d.addCallback(self.checkForOkStatus)
        # lvl 2
        d.addCallback(self.getBody)
        # lvl 3
        d.addCallback(self._assignID)

    def checkForOkStatus(self, response):
        assert response.code == self._RESPONSE_OK, "Bad response to HTTP request."
        return response

    def _assignID(self, body):
        """Download the body on successful header fetch
        from _getStrangerID
        """
        self.id = body.replace('"', '')
        ev = OmegleEvent(self.id, ID_SET, '')
        self.troll.feed(ev)  # ready to go!

    def parse_raw_events(self, events):
        """Produce OmegleEvents from a list of raw events.
        events : string
            String of raw events from a POST request to
            an omegle subpage.

        return : generator or NoneType
            Return generator with events or None if there are no events.
        """
        events = json.loads(events) or None
        if events:
            events = (OmegleEvent(self.id,
                            ev[0],
                            None if len(ev) == 1 else ev[1])
                      for ev in events)

        return events

    def getEventsPage(self):
        d = self.request('events', {'id': self.id})
        d.addCallback(self.getBody)
        d.addCallback(self.parse_raw_events)
        d.addCallback(self.troll.feed)

    def toggle_typing(self):
        def flip(resp):
            self.typing = not self.typing

        d = self.request('typing', {'id': self.id})
        d.addCallback(self.checkForOkStatus)
        d.addCallback(flip)

    def announceDisconnect(self):
        self.request('disconnect', {'id': self.id})

    def sendMessage(self, msg):
        self.request('send', {'msg': msg, 'id': self.id})



#!/usr/bin/env python
from collections import namedtuple, deque

# Omegle events
OmegleEvent = namedtuple('OmegleEvent', ['id', 'type', 'data'])
ID_SET = "idSet"
WAITING = "waiting"
CONNECTED = "connected"
TYPING = "typing"
STOPPED_TYPING = "stoppedTyping"
GOT_MESSAGE = "gotMessage"
DISCONNECTED = "strangerDisconnected"
ERROR = "error"
NULL_EVENT = OmegleEvent(None, None, None)

# Reactor events
ReactorEvent = namedtuple('ReactorEvent', ['type', 'data'])
IDLE_TIMEOUT = 'timeout'
MESSAGE_MODIFIED = 'messageModified'


def isEvent(obj):
    """Return True if object is a Tromegle event of any variety.
    """
    return hasattr(obj, '_fields')


def mkIterableSequence(obj):
    """Place obj in a tuple if obj is not a sequence container.

    Useful for handing cases where a single event can be passed to
        a function that habitually handles sequences of events.
    """
    if not hasattr(obj, '__iter__'):
        obj = (obj,)
    return obj


def spell(fn):
    """Spell decorator
    """
    if fn is not None:
        return fn
    else:
        def throwNone(out, ev):
            return None
        return throwNone


class Transmogrifier(object):
    def __init__(self, spells=()):
        self._spells = ()

        self._evQueue = None
        self.purge(spells)

    def __call__(self, events):
        """Cast all spells for each event in an iterable of events.
        """
        for ev in events:
            for spell in self._spells:
                ev = spell(self, ev)
            if ev:  # Functions may return None in order to "blackhole" an event
                self.output(ev)

    def connect(self, eventQueue):
        """Connect Transmogrifier to an eventQueue.
        """
        if not isinstance(eventQueue, deque):
            raise TypeError('Event queue must be a deque.')
        self._evQueue = eventQueue

    def push(self, spell):
        self._spells.append(spell)

    def purge(self, spells=()):
        """Remove all spells from the Transmogrifier and optionally assigns
        new ones.

        spells : tuple or list
            Spells to add (in FIFO order).
        """
        self._spells = list(mkIterableSequence(spells))

    def modifyMessage(self, ev, new_msg):
        """Take in a gotMessage event and modify its message contents.

        ev : OmegleEvent
            Must be a gotMessage event.

        new_msg : str
            String with which to replace the original message.

        return : ReactorEvent
            ReactorEvent containing old and new message.
        """
        assert ev.type == GOT_MESSAGE, "Event is not a gotMessage event."
        return ReactorEvent(MESSAGE_MODIFIED, (ev, new_msg))

    def output(self, ev):
        """Output event to the registered event queue.
        """
        assert self._evQueue is not None, 'Transmogrifier is not connected.'
        self._evQueue.append(ev)

    def levenshtein_dist(self, s1, s2):
        """
        Use:
        ====
        levenshtein_dist offers an intuitive metric for the "closeness"
        of two strings.  It is useful for catching permutation of target strings
        that differ in such aspects as capitalization and misspellings.

        Note:  for single words, a distance of 2 or 3 will catch most permutations.


        Details:
        ========
        Calculate the Levenshtein edit-distance between two strings.
        The edit distance is the number of characters that need to be
        substituted, inserted, or deleted, to transform s1 into s2.  For
        example, transforming "rain" to "shine" requires three steps,
        consisting of two substitutions and one insertion:
        "rain" -> "sain" -> "shin" -> "shine".  These operations could have
        been done in other orders, but at least three steps are needed.

        :param s1, s2: The strings to be analysed
        :type s1: str
        :type s2: str
        :rtype int
        """
        # set up a 2-D array
        len1 = len(s1)
        len2 = len(s2)
        lev = self._edit_dist_init(len1 + 1, len2 + 1)

        # iterate over the array
        for i in range(len1):
            for j in range(len2):
                self._edit_dist_step(lev, i + 1, j + 1, s1[i], s2[j])

        return lev[len1][len2]

    def _edit_dist_init(self, len1, len2):
        """from NLTK 2.0
        """
        lev = []
        for i in range(len1):
            lev.append([0] * len2)  # initialize 2-D array to zero
        for i in range(len1):
            lev[i][0] = i  # column 0: 0,1,2,3,4,...
        for j in range(len2):
            lev[0][j] = j  # row 0: 0,1,2,3,4,...
        return lev

    def _edit_dist_step(self, lev, i, j, c1, c2):
        """from NLTK 2.0
        """
        a = lev[i - 1][j] + 1  # skipping s1[i]
        b = lev[i - 1][j - 1] + (c1 != c2)  # matching s1[i] with s2[j]
        c = lev[i][j - 1] + 1  # skipping s2[j]
        lev[i][j] = min(a, b, c)  # pick the cheapest



#!/usr/bin/env python

from blessings import Terminal
from core import CBDictInterface


class Viewport(CBDictInterface):
    """Interface for printing conversations to standard output.

    ~~~DEPRECATED~~~
    """
    def __init__(self, callbackdict=None):
        super(Viewport, self).__init__(callbackdict)
        self.strangers = {}
        self.ready = 0  # no strangers connected

        print "WARNING:  This class is deprecated; It is no longer under active development and is retained as a convenience only.  Use InteractiveViewport instead."

    def on_idSet(self, ev):
        tag = 'Stranger_{0}'.format(len(self.strangers.keys()) + 1)
        self.strangers[ev.id] = tag
        print tag, "identified..."

    def on_waiting(self, ev):
        print "Waiting to connect to", self.strangers[ev.id]

    def on_connected(self, ev):
        print "Connected to", self.strangers[ev.id]
        self.ready += 1
        if self.ready == len(self.strangers.keys()):
            print
            self.ready = 0

    def on_strangerDisconnected(self, ev):
        print self.strangers[ev.id], "has disconnected."
        print
        print
        self.strangers.clear()

    def on_gotMessage(self, ev):
        print self.strangers[ev.id] + ": ", ev.data

    def on_timeout(self, ev):
        print "Idle timeout."
        self.strangers.clear()


class InteractiveViewport(CBDictInterface):
    """Interface for printing conversations, with intelligent formatting.
    """
    def __init__(self, callbackdict=None):
        super(InteractiveViewport, self).__init__(callbackdict)
        self.strangers = {}
        self.ready = 0  # no strangers connected

        self.term = Terminal()
        sc = [self.term.color(i) for i in xrange(1, 7)]
        self.strangerColLabels = tuple(sc)  # tweak order of colors
        self.strangerColors = {}

    def on_idSet(self, ev):
        tag = 'Stranger_{0}'.format(len(self.strangers.keys()) + 1)
        self.strangers[ev.id] = tag
        self.write(self.formatNotification('{0} identified...'.format(tag)))

    def on_waiting(self, ev):
        output = "Waiting to connect to " + self.strangers[ev.id]
        self.write(self.formatNotification(output))

    def on_connected(self, ev):
        output = "Connected to " + self.strangers[ev.id]
        self.write(self.formatNotification(output))

        self.ready += 1
        self.strangerColors[ev.id] = self.strangerColLabels[self.ready % len(self.strangerColLabels)]
        if self.ready == len(self.strangers.keys()):
            self.write('')
            self.ready = 0

    def on_strangerDisconnected(self, ev):
        output = self.strangers[ev.id] + " has disconnected"
        self.write(self.formatNotification(output), '')  # print empty string to skip a line
        self.strangers.clear()

    def on_error(self, ev):
        self.write(self.formatError(ev.id, ev.data))

    def on_gotMessage(self, ev):
        output = self.strangers[ev.id] + ": " + ev.data
        self.write(self.formatMessage(ev.id, output))

    def on_timeout(self, ev):
        self.write(self.formatNotification("Idle timeout."))
        self.strangers.clear()

    def on_messageModified(self, ev):
        old_ev = ev[0]
        mod_string = ev.data[1]
        mod_string, orig_string = self.formatCorrection(old_ev.id, mod_string, old_ev.data)
        self.write(mod_string, orig_string)

    def formatNotification(self, string):
        return "{t.red}{msg}{t.normal}".format(t=self.term, msg=string)

    def formatMessage(self, sid, msg):
        """Return a string with standard Omegle message formatting.

        sid : str
            Stranger id string

        msg : str
            Message body.
        """
        strngrName = self.strangers[sid]
        return "{t.bold}{color}{strngr}: {t.normal}{msg}".format(t=self.term, color=self.strangerColors[sid], strngr=strngrName, msg=msg)

    def formatCorrection(self, stranger_id, mod_string, orig_string):
        mod_string = self.formatMessage(stranger_id, mod_string)
        orig_string = "{t.cyan}{msg}{t.normal}".format(t=self.term, msg=orig_string)
        return mod_string, orig_string

    def formatError(self, sid, err_msg):
        return "{t.bold}{t.red_on_white}ERROR <{s}>: {t.normal}{t.red_on_white}{msg}{t.normal}".format(t=self.term, s=self.strangers[sid], msg=err_msg)

    def write(self, *args):
        """Print message to output.
        """
        for msg in args:
            print msg


class EventLogger(object):
    """Class to log Tromegle events for debugging purposes.
    """
    def __init__(self, logfile='tromegleEvents.log', backupCount=1, maxbytes=200):
        import logging
        import logging.handlers
        self.file = logfile
        self.logger = logging.getLogger('tromegleEvents')
        self.logger.setLevel(logging.DEBUG)
        self.handler = logging.handlers.RotatingFileHandler(self.file,
                                                            maxBytes=maxbytes,
                                                            backupCount=backupCount)
        self.logger.addHandler(self.handler)

    def notify(self, ev):
        self.logger.debug('{0}'.format(repr(ev)))



#!/usr/bin/env python

from setuptools import setup

setup(
    name='tromegle',
    version='0.1.1b alpha',
    author='Louis Thibault',
    author_email='',
    packages=['tromegle'],
    include_package_data=True,
    install_requires=['Twisted>=11.1.0', 'blessings>=1.5'],
    url='https://github.com/louist87/tromegle',
    license='GPL 3.0',
    description='Troll strangers!',
    keywords=["omegle", "trolling"],
    long_description=open('README.md').read()
)



#!/usr/bin/env python
import core
import event
import omegle
import troll
import listener

from core import startTrolling, stopTrolling

# Test imports
if __name__ == '__main__':
    m = troll.MiddleMan()
    startTrolling()
